
try:
  import sys
  import pygame
  import graphics
  import data
  import random
  from character import Character
  from scene import Scene
  import enemy
  from enemy import Enemy
except ImportError, err:
  print "couldn't load module. %s" % (err)
  sys.exit(2)

OFFSET_X = 24
PONEY_X = 450
PONEY_ADVANCED_X = 350
ROSEWIND_Y = 140
MELODY_Y = 210
MOONFIRE_Y = 280
ENEMY_X = 60
ENEMY_ADVANCED_X = 130

# current action & cursor positions
ATTACK = 0
SPECIAL = 1
ITEMS = 2
ESCAPE = 3
# cursor positions only
CHOOSING_APPLE = 4
CHOOSING_MUFFIN = 5
CHOOSING_PIE = 6
CHOOSING_CUPCAKE = 7

# action state
ADVANCING = 0
IDLE = 1
CHOOSING_TARGET = 2
CASTING = 3
RETURNING = 4
VICTORY = 5
DEFEAT = 6

# target
ROSEWIND = 10
MELODY = 11
MOONFIRE = 12
ENEMY1 = 13
ENEMY2 = 14
ENEMY3 = 15
ENEMY_ALL = 16

# animations
ANIMATION_IDLE = 0
ANIMATION_JUMP = 1
ANIMATION_WALK_RIGHT = 2
ANIMATION_ATTACK = 3
ANIMATION_SPECIAL = 4
ANIMATION_USE_ITEM = 5
ANIMATION_EXPLODE = 6
ANIMATION_FINISHED = 7

# Victory steps

class BattleScene(Scene):
  def __init__(self, game):
    Scene.__init__(self, game)
    self.initialize(game)
  
  def detectVictory(self):
    if self.numEnemies == 1:
      if self.battleEnemies[0].hp <= 0:
        return True
    elif self.numEnemies == 2:
      if self.battleEnemies[0].hp <= 0 and self.battleEnemies[1].hp <= 0:
        return True
    elif self.numEnemies == 3:
      if self.battleEnemies[0].hp <= 0 and self.battleEnemies[1].hp <= 0 and self.battleEnemies[2].hp <= 0:
        return True
    return False
  
  def winLoot(self, game):
    exp = 0
    gold = 0
    for i in range(self.numEnemies):
      exp += self.battleEnemies[i].xp
      gold += self.battleEnemies[i].gold
    
    game.player.gold += gold
    game.player.rosewind.exp += exp
    game.player.melody.exp += exp
    game.player.moonfire.exp += exp
    if game.player.rosewind.exp >= game.player.rosewind.nextLevelExp:
      game.player.rosewind.nextLevelExp += game.player.rosewind.nextLevelExp + int(0.1 * game.player.rosewind.nextLevelExp)
      game.player.rosewind.maxHp += 14
      game.player.rosewind.hp = game.player.rosewind.maxHp
      game.player.rosewind.maxMp += 6
      game.player.rosewind.mp = game.player.rosewind.maxMp
      game.player.rosewind.baseAtk += 4
      game.player.rosewind.baseMag += 2
    if game.player.melody.exp >= game.player.melody.nextLevelExp:
      game.player.melody.nextLevelExp += game.player.melody.nextLevelExp + int(0.1 * game.player.melody.nextLevelExp)
      game.player.melody.maxHp += 10
      game.player.melody.hp = game.player.melody.maxHp
      game.player.melody.maxMp += 10
      game.player.melody.mp = game.player.melody.maxMp
      game.player.melody.baseAtk += 3
      game.player.melody.baseMag += 3
    if game.player.moonfire.exp >= game.player.moonfire.nextLevelExp:
      game.player.moonfire.nextLevelExp += game.player.moonfire.nextLevelExp + int(0.1 * game.player.moonfire.nextLevelExp)
      game.player.moonfire.maxHp += 12
      game.player.moonfire.hp = game.player.moonfire.maxHp
      game.player.moonfire.maxMp += 8
      game.player.moonfire.mp = game.player.moonfire.maxMp
      game.player.moonfire.baseAtk += 3
      game.player.moonfire.baseMag += 3
      
  def initialize(self, game):
    day = 1
    self.background, self.rect = graphics.loadImage("battle" + str(day) + ".png")
    self.battleMenu, self.rect = graphics.loadImage("battlemenu.png")
    self.battleMenu2, self.rect = graphics.loadImage("battlemenu2.png")
    self.cursorImage, self.rect = graphics.loadImage("handcursor.png")
    self.cursorPosition = ATTACK
    self.action = ATTACK
    self.currentTurn = ROSEWIND
    self.currentAction = ATTACK
    self.state = ADVANCING
    self.currentTarget = ENEMY1
    self.loadEnemies()
    self.loadBattleEffects()
    # THIS WILL BE CALLED FROM OUTSIDE. FOR TESTING ONLY
    self.initializeBattle(game, day, False)
    # Use their animations only. Not their stats
    self.rosewind = Character("Rosewind", PONEY_X, ROSEWIND_Y)
    self.melody = Character("Melody", PONEY_X, MELODY_Y)
    self.moonfire = Character("Moonfire", PONEY_X, MOONFIRE_Y)
    self.initializeText(game)
    self.rosewindFrame = 0
    self.rosewindDisplayFrame = 0
    self.melodyFrame = 0
    self.melodyDisplayFrame = 0
    self.moonfireFrame = 0
    self.moonfireDisplayFrame = 0
    self.rosewindAnimation = ANIMATION_IDLE
    self.melodyAnimation = ANIMATION_IDLE
    self.moonfireAnimation = ANIMATION_IDLE
    self.lastEnemy = ENEMY1
    self.lastAlly = ROSEWIND
    self.chosenMove = -1
    self.hasUpdated = False
    
  def initializeText(self, game):
    fontFile = data.load("year_is_199x.ttf")
    self.font = pygame.font.Font(fontFile, 35)
    self.attackLabel = self.font.render("Attack",1,(255,255,255))
    self.specialLabel = self.font.render("Special  power",1,(255,255,255))
    self.itemsLabel = self.font.render("Items",1,(255,255,255))
    self.escapeLabel = self.font.render("Escape",1,(255,255,255))
    self.rosewindLabel = self.font.render("Rosewind",1,(255,255,255))
    self.melodyLabel = self.font.render("Melody",1,(255,255,255))
    self.moonfireLabel = self.font.render("Moonfire",1,(255,255,255))
    self.HPLabel = self.font.render("HP",1,(0,249,240))
    self.MPLabel = self.font.render("MP",1,(0,249,240))
    self.enemy1Name = self.font.render(self.battleEnemies[0].name,1,(255,255,255))
    if self.numEnemies >= 2:
      self.enemy2Name = self.font.render(self.battleEnemies[1].name,1,(255,255,255))
    if self.numEnemies >= 3:
      self.enemy3Name = self.font.render(self.battleEnemies[2].name,1,(255,255,255))
    fontFile.close()
    self.updateText(game)
    self.showMessage = False
  
  def updateText(self, game):
    fontFile = data.load("year_is_199x.ttf")
    self.font = pygame.font.Font(fontFile, 35)
    self.rosewindHp = self.font.render(str(game.player.rosewind.hp) + " / " + str(game.player.rosewind.maxHp),1,(255,255,255))
    self.rosewindMp = self.font.render(str(game.player.rosewind.mp) + " / " + str(game.player.rosewind.maxMp),1,(255,255,255))
    self.melodyHp = self.font.render(str(game.player.melody.hp) + " / " + str(game.player.melody.maxHp),1,(255,255,255))
    self.melodyMp = self.font.render(str(game.player.melody.mp) + " / " + str(game.player.melody.maxMp),1,(255,255,255))
    self.moonfireHp = self.font.render(str(game.player.moonfire.hp) + " / " + str(game.player.moonfire.maxHp),1,(255,255,255))
    self.moonfireMp = self.font.render(str(game.player.moonfire.mp) + " / " + str(game.player.moonfire.maxMp),1,(255,255,255))
    self.appleLabel = self.font.render("Apple  X" + str(game.player.items.consumable4),1,(255,255,255))
    self.muffinLabel = self.font.render("Muffin  X" + str(game.player.items.consumable1),1,(255,255,255))
    self.pieLabel = self.font.render("Pie  X" + str(game.player.items.consumable3),1,(255,255,255))
    self.cupcakeLabel = self.font.render("Cupcake  X" + str(game.player.items.consumable2),1,(255,255,255))
    fontFile.close()
  
  def updateBattleText(self, message):
    fontFile = data.load("kongtext.ttf")
    self.font = pygame.font.Font(fontFile, 50)
    self.battleMessage = self.font.render(message,1,(255,255,255))
    fontFile.close()
  
  def createMessage(self, message):
    if self.showMessage == False:
      fontFile = data.load("year_is_199x.ttf")
      self.font = pygame.font.Font(fontFile, 50)
      self.message = self.font.render(message,1,(255,255,255))
      fontFile.close()
      self.showMessage = True
  
  def loadEnemies(self):
    spriteset = graphics.loadTileset("enemies.png", 172, 210)
    file = data.load("enemies.info")
    i = 0
    while i < 24: # We skip 24 lines
      file.readline()
      i += 1
    self.enemies = []
    i = 0
    while i < 38:
      lines = []
      line = " "
      while line[0] != "-":
        line = file.readline()
        l = len(line)
        line = line[:l-1]
        lines.append(line)
      enemy = Enemy()
      enemy.load(lines)
      enemy.sprite = spriteset[i]
      self.enemies.append(enemy)
      i += 1
  
  def getNextEnemy(self):
    if self.battleEnemies[0].hp > 0:
      return ENEMY1
    if self.numEnemies >= 2:
      if self.battleEnemies[1].hp > 0:
        return ENEMY2
    if self.numEnemies >= 3:
      if self.battleEnemies[2].hp > 0:
        return ENEMY3
  
  def getNextAlly(self, game):
    if game.player.rosewind.hp > 0:
      return ROSEWIND
    if game.player.melody.hp > 0:
      return MELODY
    if game.player.moonfire.hp > 0:
      return MOONFIRE
        
  def loadBattleEffects(self):
    self.effect = -1
    self.effectFrame = 0
    self.effectDisplayFrame = 0
    spriteset = graphics.loadTileset("combatfx.png", 96, 80)
    self.slashSingleAnimation = spriteset[0:5]
    self.meleeSingleAnimation = spriteset[6:11]
    self.frostSingleIdle = spriteset[12]
    self.frostSingleExplosionAnimation = spriteset[13:17]
    self.frostAllIdle = spriteset[18]
    self.frostAllExplosionAnimation = spriteset[19:24]
    self.needleSingleIdle = spriteset[26]
    self.needleSingleExplosionAnimation = spriteset[27:30]
    self.poisonAllAnimation = spriteset[30:35]
    self.sunSingleAnimation = spriteset[36:42]
    self.sunAllIdle = spriteset[42]
    self.sunAllExplosionAnimation = spriteset[43:48]
    self.thunderAllAnimation = spriteset[48:54]
    self.fogAllAnimation = spriteset[54:60]
    self.windAllIdle = spriteset[60]
    self.windAllExplosionAnimation = spriteset[61:66]
    self.tentacleSingleAnimation = spriteset[66:72]
    self.tentacleAllAnimation = spriteset[72:78]
    self.fireballSingleIdle = spriteset[78]
    self.fireballSingleExplosionAnimation = spriteset[79:84]
    self.darkFireballSingleIdle = spriteset[84]
    self.darkFireballSingleExplosionAnimation = spriteset[85:90]
    self.darkFireball2SingleIdle = spriteset[90]
    self.darkFireball2SingleExplosionAnimation = spriteset[91:96]
    self.waterAllAnimation = spriteset[96:101]
    
  def initializeBattle(self, game, day, isBossBattle):
    self.initializeEnemies(day, isBossBattle)
    self.isFadingOut = True
    self.alpha = 255
  
  def initializeEnemies(self, day, isBossBattle):
    self.battleEnemies = []
    if isBossBattle == False:
      self.numEnemies = random.randint(1, 3)
      for i in range (self.numEnemies):
        self.battleEnemies.append(self.enemies[(day - 1) * 4 + random.randint(0,2)])
    else:
      if day == 10 and isBossBattle == True: # Last boss has 2 minions with him, others have none
        self.numEnemies = 3
        self.battleEnemies.append(self.enemies[36])
        self.battleEnemies.append(self.enemies[37])
        self.battleEnemies.append(self.enemies[36])
      else:
        self.numEnemies = 1
        self.battleEnemies.append(self.enemies[(day - 1) * 4 + 3])
    if self.numEnemies == 1:
      self.enemy1Y = 140
    elif self.numEnemies == 2:
      self.enemy1Y = 105
      self.enemy2Y = 175
    elif self.numEnemies == 3:
      self.enemy1Y = 70
      self.enemy2Y = 140
      self.enemy3Y = 210
    self.enemy1X = ENEMY_X
    self.enemy2X = ENEMY_X
    self.enemy3X = ENEMY_X
        
  def update(self, game):
    
    if self.detectVictory():
      self.winLoot(game)
      game.switchToWorld()
    # Update rosewind
    if self.currentTurn == ROSEWIND and game.player.rosewind.hp > 0:
      if self.state == ADVANCING:
        if self.rosewind.x > PONEY_ADVANCED_X:
          self.hasUpdated = False
          self.showMessage = False
          self.rosewindAnimation = ANIMATION_JUMP
          self.rosewind.x -= 4
          self.rosewindFrame += 1
          if self.rosewindFrame >= len(self.rosewind.jumpLeftAnimation) * 6:
            self.rosewindFrame = 0
          self.rosewindDisplayFrame = int(self.rosewindFrame / 6)
        else:
          self.state = IDLE
          self.rosewindAnimation = ANIMATION_IDLE
          self.rosewindFrame = 0
          self.rosewindDisplayFrame = 0
      elif self.state == CASTING:
        self.rosewindAnimation = ANIMATION_ATTACK
        self.rosewindFrame += 1
        if self.rosewindFrame >= len(self.rosewind.basicAttack) * 4:
          self.rosewindFrame = 0
          self.state == RETURNING
        self.rosewindDisplayFrame = int(self.rosewindFrame / 4)
      elif self.state == RETURNING:
        if self.rosewind.x < PONEY_X:
          self.rosewindAnimation = ANIMATION_WALK_RIGHT
          self.rosewind.x += 3
          self.rosewindFrame += 1
          if self.rosewindFrame >= len(self.rosewind.walkRightAnimation) * 4:
            self.rosewindFrame = 0
          self.rosewindDisplayFrame = int(self.rosewindFrame / 4)
        else:
          self.rosewindAnimation = ANIMATION_IDLE
          self.rosewindFrame = 0
          self.rosewindDisplayFrame = 0
          self.currentTurn = MELODY
          self.state = ADVANCING
    # Update melody
    elif self.currentTurn == MELODY and game.player.melody.hp > 0:
      if self.state == ADVANCING:
        if self.melody.x > PONEY_ADVANCED_X:
          self.hasUpdated = False
          self.showMessage = False
          self.melodyAnimation = ANIMATION_JUMP
          self.melody.x -= 4
          self.melodyFrame += 1
          if self.melodyFrame >= len(self.melody.jumpLeftAnimation) * 6:
            self.melodyFrame = 0
          self.melodyDisplayFrame = int(self.melodyFrame / 6)
        else:
          self.state = IDLE
          self.melodyAnimation = ANIMATION_IDLE
          self.melodyFrame = 0
          self.melodyDisplayFrame = 0
      elif self.state == CASTING:
        self.melodyAnimation = ANIMATION_ATTACK
        self.melodyFrame += 1
        if self.melodyFrame >= len(self.melody.basicAttack) * 4:
          self.melodyFrame = 0
          self.state == RETURNING
        self.melodyDisplayFrame = int(self.melodyFrame / 4)
      elif self.state == RETURNING:
        if self.melody.x < PONEY_X:
          self.melodyAnimation = ANIMATION_WALK_RIGHT
          self.melody.x += 3
          self.melodyFrame += 1
          if self.melodyFrame >= len(self.melody.walkRightAnimation) * 4:
            self.melodyFrame = 0
          self.melodyDisplayFrame = int(self.melodyFrame / 4)
        else:
          self.melodyAnimation = ANIMATION_IDLE
          self.melodyFrame = 0
          self.melodyDisplayFrame = 0
          self.currentTurn = MOONFIRE
          self.state = ADVANCING
    # Update moonfire
    elif self.currentTurn == MOONFIRE and game.player.moonfire.hp > 0:
      if self.state == ADVANCING:
        if self.moonfire.x > PONEY_ADVANCED_X:
          self.hasUpdated = False
          self.showMessage = False
          self.moonfireAnimation = ANIMATION_JUMP
          self.moonfire.x -= 4
          self.moonfireFrame += 1
          if self.moonfireFrame >= len(self.moonfire.jumpLeftAnimation) * 6:
            self.moonfireFrame = 0
          self.moonfireDisplayFrame = int(self.moonfireFrame / 6)
        else:
          self.state = IDLE
          self.moonfireAnimation = ANIMATION_IDLE
          self.moonfireFrame = 0
          self.moonfireDisplayFrame = 0
      elif self.state == CASTING:
        self.moonfireAnimation = ANIMATION_ATTACK
        self.moonfireFrame += 1
        if self.moonfireFrame >= len(self.moonfire.basicAttack) * 4:
          self.moonfireFrame = 0
          self.state == RETURNING
        self.moonfireDisplayFrame = int(self.moonfireFrame / 4)
      elif self.state == RETURNING:
        if self.moonfire.x < PONEY_X:
          self.moonfireAnimation = ANIMATION_WALK_RIGHT
          self.moonfire.x += 3
          self.moonfireFrame += 1
          if self.moonfireFrame >= len(self.moonfire.walkRightAnimation) * 4:
            self.moonfireFrame = 0
          self.moonfireDisplayFrame = int(self.moonfireFrame / 4)
        else:
          self.moonfireAnimation = ANIMATION_IDLE
          self.moonfireFrame = 0
          self.moonfireDisplayFrame = 0
          self.currentTurn = self.getNextEnemy()
          self.state = ADVANCING
    # Update enemy 1
    elif self.currentTurn == ENEMY1:
      if self.state == ADVANCING:
        if self.enemy1X < ENEMY_ADVANCED_X:
          self.hasUpdated = False
          self.showMessage = False
          self.enemy1X += 3
        else:
          self.state = IDLE
      elif self.state == IDLE: # Enemy plays its turn
        self.playEnemyTurn(ENEMY1, game)
      elif self.state == RETURNING:
        if self.enemy1X > ENEMY_X:
          self.enemy1X -= 3
        else:
          if self.numEnemies >= 2:
            self.currentTurn = ENEMY2
            self.state = ADVANCING
          else:
            self.currentTurn = ROSEWIND
            self.state = ADVANCING
            self.currentTarget = self.cursorPosition
    # Update enemy 2
    elif self.currentTurn == ENEMY2:
      if self.state == ADVANCING:
        if self.enemy2X < ENEMY_ADVANCED_X:
          self.hasUpdated = False
          self.showMessage = False
          self.enemy2X += 3
        else:
          self.state = IDLE
      elif self.state == IDLE: # Enemy plays its turn
        self.playEnemyTurn(ENEMY2, game)
      elif self.state == RETURNING:
        if self.enemy2X > ENEMY_X:
          self.enemy2X -= 3
        else:
          if self.numEnemies >= 3:
            self.currentTurn = ENEMY3
            self.state = ADVANCING
          else:
            self.currentTurn = ROSEWIND
            self.state = ADVANCING
            self.currentTarget = self.cursorPosition
    # Update enemy 3
    elif self.currentTurn == ENEMY3:
      if self.state == ADVANCING:
        if self.enemy3X < ENEMY_ADVANCED_X:
          self.hasUpdated = False
          self.showMessage = False
          self.enemy3X += 3
        else:
          self.state = IDLE
      elif self.state == IDLE: # Enemy plays its turn
        self.playEnemyTurn(ENEMY3, game)
      elif self.state == RETURNING:
        if self.enemy3X > ENEMY_X:
          self.enemy3X -= 3
        else:
          self.currentTurn = ROSEWIND
          self.state = ADVANCING
          self.currentTarget = self.cursorPosition
    # Update primary key
    if game.mykeys.keyPrimaryIsReleased == True:
      game.mykeys.keyPrimaryIsReleased = False
      if self.state == IDLE:
        if self.cursorPosition == ATTACK:
          self.action = ATTACK
          self.state = CHOOSING_TARGET
          self.cursorPosition = self.getNextEnemy()
          self.currentTarget = self.cursorPosition
        #elif self.cursorPosition == SPECIAL:
        #  self.action = SPECIAL
        #  self.state = CHOOSING_TARGET
        #  if self.currentTurn == MELODY: # Melody special heals an ally
        #    self.cursorPosition = self.getNextAlly(game)
        #    self.currentTarget = self.cursorPosition
        #  elif self.currentTurn == MOONFIRE: # Moonfire special hits all enemies
        #    self.cursorPosition = ENEMY_ALL
        #    self.currentTarget = self.cursorPosition
        #  else:
        #    self.cursorPosition = self.getNextEnemy()
        #    self.currentTarget = self.cursorPosition
        #elif self.cursorPosition == ITEMS:
        #  self.cursorPosition = CHOOSING_APPLE
        #  self.action = CHOOSING_APPLE
      # NORMAL ATTACK
      elif self.state == CHOOSING_TARGET:
        if self.action == ATTACK:
          if self.currentTurn == ROSEWIND:
            self.state = CASTING
            self.effect = enemy.SLASH_SINGLE
            if self.hasUpdated == False:
              self.hasUpdated = True
              self.createMessage("Slash")
              self.battleEnemies[self.currentTarget - ENEMY1].hp -= game.player.rosewind.atk
          elif self.currentTurn == MELODY:
            self.state = CASTING
            self.effect = enemy.MELEE_SINGLE
            if self.hasUpdated == False:
              self.hasUpdated = True
              self.createMessage("Punch")
              self.battleEnemies[self.currentTarget - ENEMY1].hp -= game.player.melody.atk
          elif self.currentTurn == MOONFIRE:
            self.state = CASTING
            self.effect = enemy.MELEE_SINGLE
            if self.hasUpdated == False:
              self.hasUpdated = True
              self.createMessage("Punch")
              self.battleEnemies[self.currentTarget - ENEMY1].hp -= game.player.moonfire.atk
        # USING SPECIAL POWER
        #elif self.action == SPECIAL:
        #  if self.currentTurn == ROSEWIND:
        #    if game.player.rosewind.mp >= game.player.rosewind.specialCost:
        #      self.state = CASTING
        #      self.effect = enemy.ROSEWIND_SPECIAL
        #    else:
        #      createMessage("Not enough MP !")
        #  elif self.currentTurn == MELODY:
        #    if self.currentTarget == ROSEWIND and game.player.rosewind.hp >= game.player.rosewind.maxHp:
        #        createMessage("Rosewind has full HP !")
        #    elif self.currentTarget == MELODY and game.player.melody.hp >= game.player.melody.maxHp:
        #        createMessage("Melody has full HP !")
        #    elif self.currentTarget == MOONFIRE and game.player.moonfire.hp >= game.player.moonfire.maxHp:
        #        createMessage("Moonfire has full HP !")
        #    else:
        #      if game.player.melody.mp >= game.player.melody.specialCost:
        #        self.state = CASTING
        #        self.effect = enemy.MELODY_SPECIAL
        #      else:
        #        createMessage("Not enough MP !")
        #  elif self.currentTurn == MOONFIRE:
        #    if game.player.moonfire.mp >= game.player.moonfire.specialCost:
        #      self.state = CASTING
        #      self.effect = enemy.MELODY_SPECIAL
        #    else:
        #      createMessage("Not enough MP !")
        # USE ITEMS
        #elif self.action == ITEMS:
        #  if self.currentTarget == ROSEWIND and game.player.rosewind.hp >= game.player.rosewind.maxHp:
        #      createMessage("Rosewind has full HP !")
        #  elif self.currentTarget == MELODY and game.player.melody.hp >= game.player.melody.maxHp:
        #      createMessage("Melody has full HP !")
        #  elif self.currentTarget == MOONFIRE and game.player.moonfire.hp >= game.player.moonfire.maxHp:
        #      createMessage("Moonfire has full HP !")
        #  else:
        #    if game.player.melody.mp >= game.player.melody.specialCost:
        #      self.state = CASTING
        #      self.effect = enemy.MELODY_SPECIAL
        #    else:
        #      createMessage("Not enough MP !")  
    # Update secondary key
    elif game.mykeys.keySecondaryIsReleased == True:
      game.mykeys.keySecondaryIsReleased = False
      if self.cursorPosition >= CHOOSING_APPLE and self.cursorPosition <= CHOOSING_CUPCAKE:
        self.action = ITEMS
        self.state = IDLE
        self.cursorPosition = ITEMS
      elif self.state == CHOOSING_TARGET:
        self.state = IDLE
        if self.action == ATTACK:
          self.cursorPosition = ATTACK
        elif self.action == SPECIAL:
          self.cursorPosition = SPECIAL
        elif self.action == ITEMS:
          self.cursorPosition = CHOOSING_APPLE
          
    # Update key up
    if game.mykeys.keyUpIsReleased == True:
      game.mykeys.keyUpIsReleased = False
      #if self.cursorPosition >= SPECIAL and self.cursorPosition <= ESCAPE:
      #  self.cursorPosition -= 1
      if self.cursorPosition >= CHOOSING_MUFFIN and self.cursorPosition <= CHOOSING_CUPCAKE:
        self.cursorPosition -= 1
      elif self.cursorPosition == MELODY or self.cursorPosition == MOONFIRE:
        self.cursorPosition -= 1
        self.currentTarget = self.cursorPosition
      elif self.cursorPosition == ENEMY2 and self.battleEnemies[0].hp > 0:
        self.cursorPosition -= 1
        self.currentTarget = self.cursorPosition
      elif self.cursorPosition == ENEMY3 and self.battleEnemies[1].hp > 0:
        self.cursorPosition -= 1
        self.currentTarget = self.cursorPosition
    # Update key down
    elif game.mykeys.keyDownIsReleased == True:
      game.mykeys.keyDownIsReleased = False
      #if self.cursorPosition >= ATTACK and self.cursorPosition <= ITEMS:
      #  self.cursorPosition += 1
      if self.cursorPosition >= CHOOSING_APPLE and self.cursorPosition <= CHOOSING_PIE:
        self.cursorPosition += 1
      elif self.cursorPosition == ROSEWIND or self.cursorPosition == MELODY:
        self.cursorPosition += 1
        self.currentTarget = self.cursorPosition
      elif self.cursorPosition == ENEMY1:
        if self.numEnemies >= 2:
          if self.battleEnemies[1].hp > 0:
            self.cursorPosition = ENEMY2
            self.currentTarget = self.cursorPosition
        elif self.numEnemies >= 3:
          if self.battleEnemies[2].hp > 0:
            self.cursorPosition = ENEMY3
            self.currentTarget = self.cursorPosition
      elif self.cursorPosition == ENEMY2 and self.numEnemies == 3:
        if self.battleEnemies[2].hp > 0:
          self.cursorPosition = ENEMY3
          self.currentTarget = self.cursorPosition
    # Consume unused key releases
    if game.mykeys.keyLeftIsReleased == True:
      game.mykeys.keyLeftIsReleased = False
    if game.mykeys.keyRightIsReleased == True:
      game.mykeys.keyRightIsReleased = False
    if self.state == CASTING:
      if self.action == ATTACK or self.action == SPECIAL:
        self.updateEffect()
    self.updateText(game)
      
  def playEnemyTurn(self, enemyNumber, game):
    k = enemyNumber - ENEMY1
    if self.hasUpdated == False:
      self.currentTarget = ROSEWIND + random.randint(0,2)
      if self.battleEnemies[k].numMoves == 1:
        self.createMessage(self.battleEnemies[k].move1Name)
        self.chosenMove = 1
      elif self.battleEnemies[k].numMoves == 2:
        maxProbability = self.battleEnemies[k].move1Probability + self.battleEnemies[k].move2Probability
        seed = random.randint(1, maxProbability)
        if seed <= self.battleEnemies[k].move1Probability:
          self.createMessage(self.battleEnemies[k].move1Name)
          self.chosenMove = 1
        else:
          self.createMessage(self.battleEnemies[k].move2Name)
          self.chosenMove = 2
      elif self.battleEnemies[k].numMoves == 3:
          maxProbability = self.battleEnemies[k].move1Probability + self.battleEnemies[k].move2Probability + self.battleEnemies[k].move3Probability
          seed = random.randint(1, maxProbability)
          if seed <= self.battleEnemies[k].move1Probability:
            self.createMessage(self.battleEnemies[k].move1Name)
            self.chosenMove = 1
          elif seed > self.battleEnemies[k].move1Probability and seed < (self.battleEnemies[k].move1Probability + self.battleEnemies[k].move2Probability):
            self.createMessage(self.battleEnemies[k].move2Name)
            self.chosenMove = 2
          else:
            self.createMessage(self.battleEnemies[k].move3Name)
            self.chosenMove = 3
      i = 0
      if self.chosenMove == 1:
        if i < self.battleEnemies[k].move1NumAttacks:
          if self.state == RETURNING:
            self.state = CASTING
            # ATTACK AGAIN
          else:
            self.action = ATTACK
            self.state = CASTING
            self.effect = self.battleEnemies[k].move1Effect
            if self.hasUpdated == False:
              self.hasUpdated = True
              dmg = random.randint(self.battleEnemies[k].move1DamageMin, self.battleEnemies[k].move1DamageMax - 1)
              if self.currentTarget == ROSEWIND:
                game.player.rosewind.hp -= dmg
              elif self.currentTarget == MELODY:
                game.player.melody.hp -= dmg
              elif self.currentTarget == MOONFIRE:
                game.player.moonfire.hp -= dmg
          i += 1
      elif self.chosenMove == 2:
        if i < self.battleEnemies[k].move2NumAttacks:
          if self.state == RETURNING:
            self.state = CASTING
            # ATTACK AGAIN
          else:
            self.action = ATTACK
            self.state = CASTING
            self.effect = self.battleEnemies[k].move2Effect
            if self.hasUpdated == False:
              self.hasUpdated = True
              dmg = random.randint(self.battleEnemies[k].move2DamageMin, self.battleEnemies[k].move2DamageMax - 1)
              if self.currentTarget == ROSEWIND:
                game.player.rosewind.hp -= dmg
              elif self.currentTarget == MELODY:
                game.player.melody.hp -= dmg
              elif self.currentTarget == MOONFIRE:
                game.player.moonfire.hp -= dmg
          i += 1
      elif self.chosenMove == 3:
        if i < self.battleEnemies[k].move2NumAttacks:
          if self.state == RETURNING:
            self.state = CASTING
            # ATTACK AGAIN
          else:
            self.action = ATTACK
            self.state = CASTING
            self.effect = self.battleEnemies[k].move3Effect
            if self.hasUpdated == False:
              self.hasUpdated = True
              dmg = random.randint(self.battleEnemies[k].move3DamageMin, self.battleEnemies[k].move3DamageMax - 1)
              if self.currentTarget == ROSEWIND:
                game.player.rosewind.hp -= dmg
              elif self.currentTarget == MELODY:
                game.player.melody.hp -= dmg
              elif self.currentTarget == MOONFIRE:
                game.player.moonfire.hp -= dmg
          i += 1
      
  def updateEffect(self):
    # SLASH_SINGLE (0)
    if self.effect == enemy.SLASH_SINGLE:
      self.effectFrame += 1
      if self.effectFrame >= len(self.slashSingleAnimation) * 3:
        self.effectFrame = 0
        self.resetBattleVars()
        return
      self.effectDisplayFrame = int(self.effectFrame / 3)
    # MELEE_SINGLE (1)
    elif self.effect == enemy.MELEE_SINGLE:
      self.effectFrame += 1
      if self.effectFrame >= len(self.meleeSingleAnimation) * 4:
        self.effectFrame = 0
        self.resetBattleVars()
        return
      self.effectDisplayFrame = int(self.effectFrame / 4)
    # FROST_SINGLE (2)
    elif self.effect == enemy.FROST_SINGLE:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(self.currentTarget)
        self.casterX = self.targetX - 500
        self.casterY = self.targetY - 400
        self.projectileX = self.casterX
        self.projectileY = self.casterY
        self.effectAnimation = ANIMATION_IDLE
        self.effectFrame += 1
      else:
        if self.projectileX > (self.targetX - 50) or self.projectileY > (self.targetY - 40):
          self.effectAnimation = ANIMATION_EXPLODE
        if self.effectAnimation == ANIMATION_EXPLODE:
          self.effectFrame += 1
          if self.effectFrame >= len(self.frostSingleExplosionAnimation) * 2:
            self.resetBattleVars()
            return
          self.effectDisplayFrame = int(self.effectFrame / 2)
        self.projectileX += 10
        self.projectileY += 8
    # FROST_ALL (3)
    elif self.effect == enemy.FROST_ALL:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(MELODY)
        self.casterX = self.targetX - 500
        self.casterY = self.targetY - 400
        self.numProjectiles = 100
        self.effectsFrame = []
        self.projectilesX = []
        self.projectilesY = []
        self.effectsDisplayFrame = []
        self.effectsAnimation = []
        for i in range (self.numProjectiles):
          self.effectsFrame.append(0)
          self.projectilesX.append(- 10 * i + self.casterX + random.randint(-50, 50))
          self.projectilesY.append(- 8 * i + self.casterY + random.randint(-100, 100))
          self.effectsDisplayFrame.append(0)
          self.effectsAnimation.append(ANIMATION_IDLE)
      else:
        for i in range (self.numProjectiles):
          self.projectilesX[i] += 10 + random.randint(-2,2)
          self.projectilesY[i] += 8 + random.randint(-2,2)
          if self.effectsAnimation[i] == ANIMATION_IDLE:
            if self.projectilesX[i] > (self.targetX - 50) or self.projectilesY[i] > (self.targetY - 40):
              self.effectsAnimation[i] = ANIMATION_EXPLODE
          elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
            self.effectsFrame[i] += 1
            self.effectsDisplayFrame[i] = int(self.effectsFrame[i] / 2)
            if self.effectsFrame[i] >= len(self.frostAllExplosionAnimation) * 2:
              self.effectsAnimation[i] = ANIMATION_FINISHED
      if self.effectsAnimation[self.numProjectiles - 1] == ANIMATION_FINISHED:
        self.resetBattleVars()
        return
      self.effectFrame += 1
    # NEEDLE_SINGLE (4)
    elif self.effect == enemy.NEEDLE_SINGLE:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(self.currentTarget)
        self.casterX, self.casterY = self.getTargetXY(self.currentTurn)
        self.numProjectiles = 35
        self.effectsFrame = []
        self.projectilesX = []
        self.projectilesY = []
        self.effectsDisplayFrame = []
        self.effectsAnimation = []
        for i in range (self.numProjectiles):
          self.effectsFrame.append(0)
          self.projectilesX.append(self.casterX + random.randint(-250, 250))
          self.projectilesY.append(self.targetY + random.randint(-40, 40))
          self.effectsDisplayFrame.append(0)
          self.effectsAnimation.append(ANIMATION_IDLE)
      else:
        for i in range (self.numProjectiles):
          self.projectilesX[i] += 14 + random.randint(-3,3)
          self.projectilesY[i] += 0
          if self.effectsAnimation[i] == ANIMATION_IDLE:
            if self.projectilesX[i] > (self.targetX + 100):
              self.effectsAnimation[i] = ANIMATION_EXPLODE
          elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
            self.effectsFrame[i] += 1
            self.effectsDisplayFrame[i] = int(self.effectsFrame[i] / 2)
            if self.effectsFrame[i] >= len(self.needleSingleExplosionAnimation) * 2:
              self.effectsAnimation[i] = ANIMATION_FINISHED
      if self.effectsAnimation[self.numProjectiles - 1] == ANIMATION_FINISHED:
        self.resetBattleVars()
        return
      self.effectFrame += 1
    # POISON_ALL (5)
    elif self.effect == enemy.POISON_ALL:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(ROSEWIND)
        self.casterX = self.targetX
        self.casterY = self.targetY + 200
        self.numProjectiles = 60
        self.effectsFrame = []
        self.projectilesX = []
        self.projectilesY = []
        self.effectsDisplayFrame = []
        self.effectsAnimation = []
        for i in range (self.numProjectiles):
          self.effectsFrame.append(0)
          self.projectilesX.append(self.casterX + random.randint(-100, 100))
          self.projectilesY.append(self.casterY + random.randint(-500, 500))
          self.effectsDisplayFrame.append(0)
          self.effectsAnimation.append(ANIMATION_IDLE)
      else:
        for i in range (self.numProjectiles):
          self.projectilesX[i] += 0
          self.projectilesY[i] -= 2
          if self.effectsAnimation[i] == ANIMATION_IDLE:
            if self.projectilesY[i] > self.targetY:
              self.effectsAnimation[i] = ANIMATION_EXPLODE
          elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
            self.effectsFrame[i] += random.randint(0,1)
            self.effectsDisplayFrame[i] = int(self.effectsFrame[i] / 10)
            if self.effectsFrame[i] >= len(self.poisonAllAnimation) * 10:
              self.effectsAnimation[i] = ANIMATION_FINISHED
      if self.effectsAnimation[self.numProjectiles - 1] == ANIMATION_FINISHED or self.effectFrame > 215:
        self.resetBattleVars()
        return
      self.effectFrame += 1
    # SUN_SINGLE (6)
    elif self.effect == enemy.SUN_SINGLE:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(self.currentTarget)
        self.casterX, self.casterY = self.getTargetXY(self.currentTurn)
        self.casterX -= 1000
        self.numProjectiles = 50
        self.effectsFrame = []
        self.projectilesX = []
        self.projectilesY = []
        self.effectsDisplayFrame = []
        self.effectsAnimation = []
        for i in range (self.numProjectiles):
          self.effectsFrame.append(0)
          self.projectilesX.append(self.casterX + random.randint(-250, 250))
          self.projectilesY.append(self.targetY + random.randint(-40, 40))
          self.effectsDisplayFrame.append(0)
          self.effectsAnimation.append(ANIMATION_EXPLODE)
      else:
        for i in range (self.numProjectiles):
          self.projectilesX[i] += 25 + random.randint(-3,3)
          self.projectilesY[i] += 0
          if self.effectsAnimation[i] == ANIMATION_EXPLODE:
            if self.projectilesX[i] > 620:
              self.effectsAnimation[i] = ANIMATION_FINISHED
          elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
            self.effectsFrame[i] += 1
            self.effectsDisplayFrame[i] = int(self.effectsFrame[i] / 2)
            if self.effectsFrame[i] >= len(self.sunSingleAnimation) * 2:
              self.effectsAnimation[i] = ANIMATION_FINISHED
      if self.effectsAnimation[self.numProjectiles - 1] == ANIMATION_FINISHED:
        self.resetBattleVars()
        return
      self.effectFrame += 1
    # SUN_ALL (7)
    elif self.effect == enemy.SUN_ALL:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(MELODY)
        self.casterX = self.targetX - 500
        self.casterY = self.targetY - 400
        self.numProjectiles = 100
        self.effectsFrame = []
        self.projectilesX = []
        self.projectilesY = []
        self.effectsDisplayFrame = []
        self.effectsAnimation = []
        for i in range (self.numProjectiles):
          self.effectsFrame.append(0)
          self.projectilesX.append(- 10 * i + self.casterX + random.randint(-100, 100))
          self.projectilesY.append(- 8 * i + self.casterY + random.randint(-100, 100))
          self.effectsDisplayFrame.append(0)
          self.effectsAnimation.append(ANIMATION_IDLE)
      else:
        for i in range (self.numProjectiles):
          self.projectilesX[i] += 5 + random.randint(-1,1)
          self.projectilesY[i] += 4 + random.randint(-1,1)
          if self.effectsAnimation[i] == ANIMATION_IDLE:
            if self.projectilesX[i] > (self.targetX - 50) or self.projectilesY[i] > (self.targetY - 40):
              self.effectsAnimation[i] = ANIMATION_EXPLODE
          elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
            self.effectsFrame[i] += 1
            self.effectsDisplayFrame[i] = int(self.effectsFrame[i] / 3)
            if self.effectsFrame[i] >= len(self.sunAllExplosionAnimation) * 3:
              self.effectsAnimation[i] = ANIMATION_FINISHED
      if self.effectsAnimation[self.numProjectiles - 1] == ANIMATION_FINISHED:
        self.resetBattleVars()
        return
      self.effectFrame += 1
    # THUNDER_ALL (8)
    elif self.effect == enemy.THUNDER_ALL:
      self.effectFrame += 1
      if self.effectFrame >= len(self.thunderAllAnimation) * 4:
        self.effectFrame = 0
        self.resetBattleVars()
        return
      self.effectDisplayFrame = int(self.effectFrame / 4)
    # FOG_ALL (9)
    elif self.effect == enemy.FOG_ALL:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(ROSEWIND)
        self.casterX = self.targetX
        self.casterY = self.targetY + 200
        self.numProjectiles = 60
        self.effectsFrame = []
        self.projectilesX = []
        self.projectilesY = []
        self.effectsDisplayFrame = []
        self.effectsAnimation = []
        for i in range (self.numProjectiles):
          self.effectsFrame.append(0)
          self.projectilesX.append(self.casterX + random.randint(-100, 100))
          self.projectilesY.append(self.casterY + random.randint(-500, 500))
          self.effectsDisplayFrame.append(0)
          self.effectsAnimation.append(ANIMATION_IDLE)
      else:
        for i in range (self.numProjectiles):
          self.projectilesX[i] += 0
          self.projectilesY[i] -= 2
          if self.effectsAnimation[i] == ANIMATION_IDLE:
            if self.projectilesY[i] > self.targetY:
              self.effectsAnimation[i] = ANIMATION_EXPLODE
          elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
            self.effectsFrame[i] += random.randint(0,1)
            self.effectsDisplayFrame[i] = int(self.effectsFrame[i] / 10)
            if self.effectsFrame[i] >= len(self.fogAllAnimation) * 10:
              self.effectsAnimation[i] = ANIMATION_FINISHED
      if self.effectsAnimation[self.numProjectiles - 1] == ANIMATION_FINISHED or self.effectFrame > 215:
        self.resetBattleVars()
        return
      self.effectFrame += 1
    # WIND_ALL (10)
    elif self.effect == enemy.WIND_ALL:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(MELODY)
        self.casterX = self.targetX - 500
        self.casterY = self.targetY - 200
        self.numProjectiles = 100
        self.effectsFrame = []
        self.projectilesX = []
        self.projectilesY = []
        self.effectsDisplayFrame = []
        self.effectsAnimation = []
        for i in range (self.numProjectiles):
          self.effectsFrame.append(0)
          self.projectilesX.append(- 10 * i + self.casterX + random.randint(-50, 50))
          self.projectilesY.append(- 4 * i + self.casterY + random.randint(-100, 100))
          self.effectsDisplayFrame.append(0)
          self.effectsAnimation.append(ANIMATION_IDLE)
      else:
        for i in range (self.numProjectiles):
          self.projectilesX[i] += 10 + random.randint(-2,2)
          self.projectilesY[i] += 4 + random.randint(-2,2)
          if self.effectsAnimation[i] == ANIMATION_IDLE:
            if self.projectilesX[i] > (self.targetX):
              self.effectsAnimation[i] = ANIMATION_EXPLODE
          elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
            self.effectsFrame[i] += 1
            self.effectsDisplayFrame[i] = int(self.effectsFrame[i] / 2)
            if self.effectsFrame[i] >= len(self.windAllExplosionAnimation) * 2:
              self.effectsAnimation[i] = ANIMATION_FINISHED
      if self.effectsAnimation[self.numProjectiles - 1] == ANIMATION_FINISHED:
        self.resetBattleVars()
        return
      self.effectFrame += 1
    # TENTACLE_SINGLE (11)
    elif self.effect == enemy.TENTACLE_SINGLE:
      self.effectFrame += 1
      if self.effectFrame >= len(self.tentacleSingleAnimation) * 20:
        self.effectFrame = 0
        self.resetBattleVars()
        return
      self.effectDisplayFrame = int(self.effectFrame / 20)
    # TENTACLE_ALL (12)
    elif self.effect == enemy.TENTACLE_ALL:
      self.effectFrame += 1
      if self.effectFrame >= len(self.tentacleAllAnimation) * 20:
        self.effectFrame = 0
        self.resetBattleVars()
        return
      self.effectDisplayFrame = int(self.effectFrame / 20)
    # FIREBALL_SINGLE (13)
    if self.effect == enemy.FIREBALL_SINGLE:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(self.currentTarget)
        self.casterX, self.casterY = self.getTargetXY(self.currentTurn)
        self.projectileX = self.casterX + 20
        self.projectileY = self.casterY + 20
        self.dx = int((self.targetX - self.casterX) / 65)
        self.dy = int((self.targetY - self.casterY) / 65)
        self.effectAnimation = ANIMATION_IDLE
        self.effectFrame += 1
      else:
        if self.projectileX > (self.targetX - 25):
          self.effectAnimation = ANIMATION_EXPLODE
        if self.effectAnimation == ANIMATION_EXPLODE:
          self.effectFrame += 1
          if self.effectFrame >= len(self.fireballSingleExplosionAnimation) * 2:
            self.resetBattleVars()
            return
          self.effectDisplayFrame = int(self.effectFrame / 2)
        self.projectileX += self.dx
        self.projectileY += self.dy
    # DARK1_SINGLE (14)
    if self.effect == enemy.DARK1_SINGLE:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(self.currentTarget)
        self.casterX, self.casterY = self.getTargetXY(self.currentTurn)
        self.projectileX = self.casterX + 20
        self.projectileY = self.casterY + 20
        self.dx = int((self.targetX - self.casterX) / 40)
        self.dy = int((self.targetY - self.casterY) / 40)
        self.effectAnimation = ANIMATION_IDLE
        self.effectFrame += 1
      else:
        if self.projectileX > (self.targetX - 50):
          self.effectAnimation = ANIMATION_EXPLODE
        if self.effectAnimation == ANIMATION_EXPLODE:
          self.effectFrame += 1
          if self.effectFrame >= len(self.darkFireballSingleExplosionAnimation) * 2:
            self.resetBattleVars()
            return
          self.effectDisplayFrame = int(self.effectFrame / 2)
        self.projectileX += self.dx
        self.projectileY += self.dy
    # DARK2_SINGLE (15)
    if self.effect == enemy.DARK2_SINGLE:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(self.currentTarget)
        self.casterX, self.casterY = self.getTargetXY(self.currentTurn)
        self.casterX += 75
        self.projectileX = self.casterX
        self.projectileY = self.casterY + 10
        self.effectAnimation = ANIMATION_IDLE
        self.effectFrame += 1
      else:
        if self.effectAnimation == ANIMATION_EXPLODE:
          self.effectFrame += 1
          if self.effectFrame >= len(self.darkFireballSingleExplosionAnimation) * 2:
            self.resetBattleVars()
            return
          self.effectDisplayFrame = int(self.effectFrame / 2)
        if self.projectileX < (self.casterX + 80) and self.effectAnimation == ANIMATION_IDLE:
          self.projectileX += 1
        elif self.projectileX >= (self.casterX + 80) and self.projectileX < (self.targetX - 5) and self.projectileY > -100 and self.effectAnimation == ANIMATION_IDLE:
          self.dy += 2
          self.projectileY -= self.dy
        elif self.projectileX < (self.targetX - 5) and self.projectileY <= -100 and self.effectAnimation == ANIMATION_IDLE:
          self.projectileX += 2
        else:
          self.projectileY += 15
          if self.projectileX >= (self.targetX - 5) and self.projectileY > (self.targetY - 30):
            self.effectAnimation = ANIMATION_EXPLODE
    # WATER_ALL (16)
    elif self.effect == enemy.WATER_ALL:
      if self.effectFrame == 0:
        self.targetX, self.targetY = self.getTargetXY(ROSEWIND)
        self.casterX = self.targetX
        self.casterY = self.targetY + 200
        self.numProjectiles = 100
        self.effectsFrame = []
        self.projectilesX = []
        self.projectilesY = []
        self.effectsDisplayFrame = []
        self.effectsAnimation = []
        for i in range (self.numProjectiles):
          self.effectsFrame.append(0)
          self.projectilesX.append(self.casterX + random.randint(-100, 100))
          self.projectilesY.append(self.casterY + random.randint(-500, 500))
          self.effectsDisplayFrame.append(0)
          self.effectsAnimation.append(ANIMATION_IDLE)
      else:
        for i in range (self.numProjectiles):
          self.projectilesX[i] += 0
          self.projectilesY[i] -= 2
          if self.effectsAnimation[i] == ANIMATION_IDLE:
            if self.projectilesY[i] > self.targetY:
              self.effectsAnimation[i] = ANIMATION_EXPLODE
          elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
            self.effectsFrame[i] += random.randint(0,1)
            self.effectsDisplayFrame[i] = int(self.effectsFrame[i] / 10)
            if self.effectsFrame[i] >= len(self.waterAllAnimation) * 10:
              self.effectsAnimation[i] = ANIMATION_FINISHED
      if self.effectsAnimation[self.numProjectiles - 1] == ANIMATION_FINISHED or self.effectFrame > 215:
        self.resetBattleVars()
        return
      self.effectFrame += 1 
    
  def resetBattleVars(self):
    self.state = RETURNING
    self.action = ATTACK
    self.cursorPosition = ATTACK
    self.effect = -1
    self.showMessage = False
    self.effectsFrame = []
    self.effectFrame = 0
    self.projectilesX = []
    self.projectileX = -1
    self.projectilesY = []
    self.projectileY = -1
    self.effectsDisplayFrame = []
    self.effectAnimation = -1
    self.effectsAnimation = []
    self.dx = 0
    self.dy = 0
  
  def displayEffect(self):
    # SLASH_SINGLE (0)
    if self.effect == enemy.SLASH_SINGLE:
      x, y = self.getTargetXY(self.currentTarget)
      self.screen.blit(self.slashSingleAnimation[self.effectDisplayFrame], (x, y))
       # TODO: APPLY HIT DAMAGE : BUT NOT HERE LOL
    # MELEE_SINGLE (1)
    elif self.effect == enemy.MELEE_SINGLE:
      x, y = self.getTargetXY(self.currentTarget)
      self.screen.blit(self.meleeSingleAnimation[self.effectDisplayFrame], (x, y))
       # TODO: APPLY HIT DAMAGE
    # FROST_SINGLE (2)
    elif self.effect == enemy.FROST_SINGLE and self.effectAnimation == ANIMATION_IDLE:
      self.screen.blit(self.frostSingleIdle, (self.projectileX, self.projectileY))
    elif self.effect == enemy.FROST_SINGLE and self.effectAnimation == ANIMATION_EXPLODE:
      self.screen.blit(self.frostSingleExplosionAnimation[self.effectDisplayFrame], (self.projectileX, self.projectileY))
    # FROST_ALL (3)
    elif self.effect == enemy.FROST_ALL:
      for i in range (self.numProjectiles):
        if self.effectsAnimation[i] == ANIMATION_IDLE:
          self.screen.blit(self.frostAllIdle, (self.projectilesX[i], self.projectilesY[i]))
        elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
          self.screen.blit(self.frostAllExplosionAnimation[self.effectsDisplayFrame[i]], (self.projectilesX[i], self.projectilesY[i]))
    # NEEDLE_SINGLE (4)
    elif self.effect == enemy.NEEDLE_SINGLE:
      for i in range (self.numProjectiles):
        if self.effectsAnimation[i] == ANIMATION_IDLE:
          self.screen.blit(self.needleSingleIdle, (self.projectilesX[i], self.projectilesY[i]))
        elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
          self.screen.blit(self.needleSingleExplosionAnimation[self.effectsDisplayFrame[i]], (self.projectilesX[i], self.projectilesY[i]))
    # POISON_ALL (5)
    elif self.effect == enemy.POISON_ALL:
      for i in range (self.numProjectiles):
        if self.effectsAnimation[i] == ANIMATION_EXPLODE:
          self.screen.blit(self.poisonAllAnimation[self.effectsDisplayFrame[i]], (self.projectilesX[i], self.projectilesY[i]))
    # SUN_SINGLE (6)
    elif self.effect == enemy.SUN_SINGLE:
      for i in range (self.numProjectiles):
        if self.effectsAnimation[i] == ANIMATION_EXPLODE:
          self.screen.blit(self.sunSingleAnimation[self.effectsDisplayFrame[i]], (self.projectilesX[i], self.projectilesY[i]))
    # SUN_ALL (7)
    elif self.effect == enemy.SUN_ALL:
      for i in range (self.numProjectiles):
        if self.effectsAnimation[i] == ANIMATION_IDLE:
          self.screen.blit(self.sunAllIdle, (self.projectilesX[i], self.projectilesY[i]))
        elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
          self.screen.blit(self.sunAllExplosionAnimation[self.effectsDisplayFrame[i]], (self.projectilesX[i], self.projectilesY[i]))
    # THUNDER_ALL (8)
    elif self.effect == enemy.THUNDER_ALL:
      x, y = self.getTargetXY(ROSEWIND)
      self.screen.blit(self.tentacleSingleAnimation[self.effectDisplayFrame], (x, y))
      x, y = self.getTargetXY(MELODY)
      self.screen.blit(self.tentacleSingleAnimation[self.effectDisplayFrame], (x, y))
      x, y = self.getTargetXY(MOONFIRE)
      self.screen.blit(self.tentacleSingleAnimation[self.effectDisplayFrame], (x, y))
    # FOG_ALL (9)
    elif self.effect == enemy.FOG_ALL:
      for i in range (self.numProjectiles):
        if self.effectsAnimation[i] == ANIMATION_EXPLODE:
          self.screen.blit(self.fogAllAnimation[self.effectsDisplayFrame[i]], (self.projectilesX[i], self.projectilesY[i]))
    # WIND_ALL (10)
    elif self.effect == enemy.WIND_ALL:
      for i in range (self.numProjectiles):
        if self.effectsAnimation[i] == ANIMATION_IDLE:
          self.screen.blit(self.windAllIdle, (self.projectilesX[i], self.projectilesY[i]))
        elif self.effectsAnimation[i] == ANIMATION_EXPLODE:
          self.screen.blit(self.windAllExplosionAnimation[self.effectsDisplayFrame[i]], (self.projectilesX[i], self.projectilesY[i]))
    # TENTACLE_SINGLE (11)
    elif self.effect == enemy.TENTACLE_SINGLE:
      x, y = self.getTargetXY(self.currentTarget)
      self.screen.blit(self.tentacleSingleAnimation[self.effectDisplayFrame], (x, y))
    # TENTACLE_ALL (12)
    elif self.effect == enemy.TENTACLE_ALL:
      x, y = self.getTargetXY(ROSEWIND)
      self.screen.blit(self.tentacleAllAnimation[self.effectDisplayFrame], (x, y))
      x, y = self.getTargetXY(MELODY)
      self.screen.blit(self.tentacleAllAnimation[self.effectDisplayFrame], (x, y))
      x, y = self.getTargetXY(MOONFIRE)
      self.screen.blit(self.tentacleAllAnimation[self.effectDisplayFrame], (x, y))
    # FIREBALL_SINGLE (13)
    elif self.effect == enemy.FIREBALL_SINGLE and self.effectAnimation == ANIMATION_IDLE:
      self.screen.blit(self.fireballSingleIdle, (self.projectileX, self.projectileY))
    elif self.effect == enemy.FIREBALL_SINGLE and self.effectAnimation == ANIMATION_EXPLODE:
      self.screen.blit(self.fireballSingleExplosionAnimation[self.effectDisplayFrame], (self.projectileX, self.projectileY))
    # DARK1_SINGLE (14)
    elif self.effect == enemy.DARK1_SINGLE and self.effectAnimation == ANIMATION_IDLE:
      self.screen.blit(self.darkFireballSingleIdle, (self.projectileX, self.projectileY))
    elif self.effect == enemy.DARK1_SINGLE and self.effectAnimation == ANIMATION_EXPLODE:
      self.screen.blit(self.darkFireballSingleExplosionAnimation[self.effectDisplayFrame], (self.projectileX, self.projectileY))
    # DARK2_SINGLE (15)
    elif self.effect == enemy.DARK2_SINGLE and self.effectAnimation == ANIMATION_IDLE:
      self.screen.blit(self.darkFireball2SingleIdle, (self.projectileX, self.projectileY))
    elif self.effect == enemy.DARK2_SINGLE and self.effectAnimation == ANIMATION_EXPLODE:
      self.screen.blit(self.darkFireball2SingleExplosionAnimation[self.effectDisplayFrame], (self.projectileX, self.projectileY))
    # WATER_ALL (16)
    elif self.effect == enemy.WATER_ALL:
      for i in range (self.numProjectiles):
        if self.effectsAnimation[i] == ANIMATION_EXPLODE:
          self.screen.blit(self.waterAllAnimation[self.effectsDisplayFrame[i]], (self.projectilesX[i], self.projectilesY[i]))
        # TODO: APPLY HIT DAMAGE
#SLASH_SINGLE = 0
#MELEE_SINGLE = 1
#FROST_SINGLE = 2
#FROST_ALL = 3
#NEEDLE_SINGLE = 4
#POISON_ALL = 5
#SUN_SINGLE = 6
#SUN_ALL = 7
#THUNDER_ALL = 8
#FOG_ALL = 9
#WIND_ALL = 10
#TENTACLE_SINGLE = 11
#TENTACLE_ALL = 12
#FIREBALL_SINGLE = 13
#DARK1_SINGLE = 14
#DARK2_SINGLE = 15
#WATER_ALL = 16
  def getTargetXY(self, target):
    offsetX = 30
    offsetY = 50
    if target == ROSEWIND:
      return self.rosewind.x, self.rosewind.y
    elif target == MELODY:
      return self.melody.x, self.melody.y
    elif target == MOONFIRE:
      return self.moonfire.x, self.moonfire.y
    elif target == ENEMY1:
      return ENEMY_X + offsetX, self.enemy1Y + offsetY
    elif target == ENEMY2:
      return ENEMY_X + offsetX, self.enemy2Y + offsetY
    elif target == ENEMY3:
      return ENEMY_X + offsetX, self.enemy3Y + offsetY
      
  def render(self):
    # Blit background
    self.screen.blit(self.background, (0, 0))
    # Blit poneys
    # Rosewind animations
    if self.rosewindAnimation == ANIMATION_IDLE:
      self.screen.blit(self.rosewind.idleLeft, (self.rosewind.x, self.rosewind.y))
    elif self.rosewindAnimation == ANIMATION_JUMP:
      self.screen.blit(self.rosewind.jumpLeftAnimation[self.rosewindDisplayFrame], (self.rosewind.x, self.rosewind.y))
    elif self.rosewindAnimation == ANIMATION_WALK_RIGHT:
      self.screen.blit(self.rosewind.walkRightAnimation[self.rosewindDisplayFrame], (self.rosewind.x, self.rosewind.y))
    elif self.rosewindAnimation == ANIMATION_ATTACK:
      self.screen.blit(self.rosewind.basicAttack[self.rosewindDisplayFrame], (self.rosewind.x, self.rosewind.y))
    elif self.rosewindAnimation == ANIMATION_SPECIAL:
      self.screen.blit(self.rosewind.specialAttack[self.rosewindDisplayFrame], (self.rosewind.x, self.rosewind.y))
    # Melody animations
    if self.melodyAnimation == ANIMATION_IDLE:
      self.screen.blit(self.melody.idleLeft, (self.melody.x, self.melody.y))
    elif self.melodyAnimation == ANIMATION_JUMP:
      self.screen.blit(self.melody.jumpLeftAnimation[self.melodyDisplayFrame], (self.melody.x, self.melody.y))
    elif self.melodyAnimation == ANIMATION_WALK_RIGHT:
      self.screen.blit(self.melody.walkRightAnimation[self.melodyDisplayFrame], (self.melody.x, self.melody.y))
    elif self.melodyAnimation == ANIMATION_ATTACK:
      self.screen.blit(self.melody.basicAttack[self.melodyDisplayFrame], (self.melody.x, self.melody.y))
    elif self.melodyAnimation == ANIMATION_SPECIAL:
      self.screen.blit(self.melody.basicAttack[self.melodyDisplayFrame], (self.melody.x, self.melody.y))
    # Moonfire animations
    if self.moonfireAnimation == ANIMATION_IDLE:
      self.screen.blit(self.moonfire.idleLeft, (self.moonfire.x, self.moonfire.y))
    elif self.moonfireAnimation == ANIMATION_JUMP:
      self.screen.blit(self.moonfire.jumpLeftAnimation[self.moonfireDisplayFrame], (self.moonfire.x, self.moonfire.y))
    elif self.moonfireAnimation == ANIMATION_WALK_RIGHT:
      self.screen.blit(self.moonfire.walkRightAnimation[self.moonfireDisplayFrame], (self.moonfire.x, self.moonfire.y))
    elif self.moonfireAnimation == ANIMATION_ATTACK:
      self.screen.blit(self.moonfire.basicAttack[self.moonfireDisplayFrame], (self.moonfire.x, self.moonfire.y))
    elif self.moonfireAnimation == ANIMATION_SPECIAL:
      self.screen.blit(self.moonfire.basicAttack[self.moonfireDisplayFrame], (self.moonfire.x, self.moonfire.y))
    # Blit enemies
    if self.battleEnemies[0].hp > 0:
      self.screen.blit(self.battleEnemies[0].sprite, (self.enemy1X, self.enemy1Y))
    if self.numEnemies >= 2 and self.battleEnemies[1].hp > 0:
      self.screen.blit(self.battleEnemies[1].sprite, (self.enemy2X, self.enemy2Y))
    if self.numEnemies >= 3 and self.battleEnemies[2].hp > 0:
      self.screen.blit(self.battleEnemies[2].sprite, (self.enemy3X, self.enemy3Y))
    # Blit effects
    if self.action == ATTACK or self.action == SPECIAL and self.state == CASTING:
      self.displayEffect()
    # Show message
    if self.showMessage == True:
      self.screen.blit(self.battleMenu2, (OFFSET_X, 5))
      self.screen.blit(self.message, (OFFSET_X + 8, 15))
    # Blit battle menu
    self.screen.blit(self.battleMenu, (OFFSET_X, 368))
    if self.action >= CHOOSING_APPLE and self.action <= CHOOSING_CUPCAKE:
      self.displayItems()
    else:
      self.displayOptions()
    self.screen.blit(self.rosewindLabel, (312 + OFFSET_X, 380))
    self.screen.blit(self.melodyLabel, (312 + OFFSET_X, 410))
    self.screen.blit(self.moonfireLabel, (312 + OFFSET_X, 440))
    self.screen.blit(self.HPLabel, (385 + OFFSET_X, 380))
    self.screen.blit(self.HPLabel, (385 + OFFSET_X, 410))
    self.screen.blit(self.HPLabel, (385 + OFFSET_X, 440))
    self.screen.blit(self.rosewindHp, (405 + OFFSET_X, 380))
    self.screen.blit(self.melodyHp, (405 + OFFSET_X, 410))
    self.screen.blit(self.moonfireHp, (405 + OFFSET_X, 440))
    self.screen.blit(self.MPLabel, (490 + OFFSET_X, 380))
    self.screen.blit(self.MPLabel, (490 + OFFSET_X, 410))
    self.screen.blit(self.MPLabel, (490 + OFFSET_X, 440))
    self.screen.blit(self.rosewindMp, (516 + OFFSET_X, 380))
    self.screen.blit(self.melodyMp, (516 + OFFSET_X, 410))
    self.screen.blit(self.moonfireMp, (516 + OFFSET_X, 440))
    self.screen.blit(self.enemy1Name, (10 + OFFSET_X, 380))
    if self.numEnemies >= 2:
      self.screen.blit(self.enemy2Name, (10 + OFFSET_X, 410))
    if self.numEnemies >= 3:
      self.screen.blit(self.enemy3Name, (10 + OFFSET_X, 440))
    # Cursor positions
    if self.cursorPosition == ATTACK or self.cursorPosition == CHOOSING_APPLE:
      self.screen.blit(self.cursorImage, (145 + OFFSET_X, 375))
    elif self.cursorPosition == SPECIAL or self.cursorPosition == CHOOSING_MUFFIN:
      self.screen.blit(self.cursorImage, (145 + OFFSET_X, 400))
    elif self.cursorPosition == ITEMS or self.cursorPosition == CHOOSING_PIE:
      self.screen.blit(self.cursorImage, (145 + OFFSET_X, 425))
    elif self.cursorPosition == ESCAPE or self.cursorPosition == CHOOSING_CUPCAKE:
      self.screen.blit(self.cursorImage, (145 + OFFSET_X, 450))
    elif self.cursorPosition == ROSEWIND:
      self.screen.blit(self.cursorImage, (420 + OFFSET_X, self.rosewind.y + 30))
    elif self.cursorPosition == MELODY:
      self.screen.blit(self.cursorImage, (320 + OFFSET_X, self.melody.y + 30))
    elif self.cursorPosition == MOONFIRE:
      self.screen.blit(self.cursorImage, (420 + OFFSET_X, self.moonfire.y + 30))
    elif self.cursorPosition == ENEMY1:
      if self.numEnemies == 1:
        self.screen.blit(self.cursorImage, (70 + OFFSET_X, 245))
      elif self.numEnemies == 2:
        self.screen.blit(self.cursorImage, (70 + OFFSET_X, 210))
      elif self.numEnemies == 3:
        self.screen.blit(self.cursorImage, (70 + OFFSET_X, 175))
    elif self.cursorPosition == ENEMY2:
      if self.numEnemies == 2:
        self.screen.blit(self.cursorImage, (70 + OFFSET_X, 280))
      elif self.numEnemies == 3:
        self.screen.blit(self.cursorImage, (70 + OFFSET_X, 245))
    elif self.cursorPosition == ENEMY3:
      self.screen.blit(self.cursorImage, (70 + OFFSET_X, 315))
    # Blit screen transitions
    if self.isFadingOut == True:
      self.fadeOut()
    elif self.isFadingIn == True:
      self.fadeIn()
    pygame.display.flip()
  
  def displayOptions(self):
    self.screen.blit(self.attackLabel, (178 + OFFSET_X, 373))
    #self.screen.blit(self.specialLabel, (178 + OFFSET_X, 398))
    #self.screen.blit(self.itemsLabel, (178 + OFFSET_X, 423))
    #self.screen.blit(self.escapeLabel, (178 + OFFSET_X, 448))
  
  def displayItems(self):
    self.screen.blit(self.appleLabel, (178 + OFFSET_X, 373))
    self.screen.blit(self.muffinLabel, (178 + OFFSET_X, 398))
    self.screen.blit(self.pieLabel, (178 + OFFSET_X, 423))
    self.screen.blit(self.cupcakeLabel, (178 + OFFSET_X, 448))
    