
try:
  import sys
  import pygame
  import data
  import graphics
  import level
  from level import Level
except ImportError, err:
  print "couldn't load module. %s" % (err)
  sys.exit(2)

LEFT = 0
UP = 1
RIGHT = 2
DOWN = 3
  
class Character:
  
  def __init__(self, name, x, y):
    self.name = name
    self.isNPC = False
    self.x = x
    self.y = y
    self.isWalking = False
    self.isJumping = False
    self.isUnconscious = False
    self.isUsingItem = False
    self.isParading = False
    self.isBasicAttacking = False
    self.isSpecialAttacking = False
    self.direction = LEFT
    self.walkDirection = LEFT
    self.walkingOffsetX = 0
    self.walkingOffsetY = 0
    self.maxDistance = 0
    self.loadAnimations(self.name + ".png")
    self.level = 1
    self.boostHp = 0
    self.hp = 50
    self.boostMp = 0
    self.mp = 20
    self.baseMaxHp = 50
    self.maxHp = self.baseMaxHp + self.boostHp
    self.baseMaxMp = 20
    self.maxMp = self.baseMaxMp + self.boostMp
    self.exp = 1
    self.nextLevelExp = 100
    self.baseAtk = 3
    self.boostAtk = 0
    self.atk = self.baseAtk + self.boostAtk
    self.baseMag = 3
    self.boostMag = 0
    if name == "Melody":
      self.specialCost = int(self.level / 4) + 3
    else:
      self.specialCost = int(self.level / 3) + 4
    self.mag = self.baseMag + self.boostMag
    self.restoredHPAfterFight = 0
    self.restoredMPAfterFight = 0
    
  def loadAnimations(self, filename):
    self.animationFrame = 0
    spriteset = graphics.loadPoneySprites(filename)
    self.idleLeft = spriteset[0]
    self.walkLeftAnimation = spriteset[1:7]
    self.idleRight = spriteset[0]
    self.idleRight = pygame.transform.flip(self.idleRight, 1, 0)
    self.walkRightAnimation = spriteset[1:7]
    for frame in range(6):
      self.walkRightAnimation[frame] = pygame.transform.flip(self.walkRightAnimation[frame], 1, 0)
    self.idleUp = spriteset[40]
    self.walkUpAnimation = spriteset[41:47]
    self.idleDown = spriteset[20]
    self.walkDownAnimation = spriteset[21:27]
    self.jumpLeftAnimation = spriteset[7:11]
    self.jumpRightAnimation = spriteset[7:11]
    for frame in range(4):
      self.jumpRightAnimation[frame] = pygame.transform.flip(self.jumpRightAnimation[frame], 1, 0)
    self.useItemAnimation = spriteset[11:13]
    self.unconscious = spriteset[13]
    if self.name == "Moonfire":
      self.basicAttack = spriteset[27:29]
    else:
      self.basicAttack = spriteset[27:31]
    self.specialAttack = spriteset[47:50]
    if self.name == "Melody":
      self.maxDistance = 40
    elif self.name == "Moonfire":
      self.maxDistance = 80
  
  def updateWalkingOffsets(self):
    if self.name == "Melody" or self.name == "Moonfire":
      if self.direction == UP:
        if self.walkingOffsetY < self.maxDistance:
          self.walkingOffsetY += 3
        if self.walkingOffsetX > 0:
          self.walkingOffsetX -= 3
        elif self.walkingOffsetX < 0:
          self.walkingOffsetX += 3
      if self.direction == DOWN:
        if self.walkingOffsetY > -self.maxDistance:
          self.walkingOffsetY -= 3
        if self.walkingOffsetX > 0:
          self.walkingOffsetX -= 3
        elif self.walkingOffsetX < 0:
          self.walkingOffsetX += 3
      if self.direction == LEFT:
        if self.walkingOffsetX < self.maxDistance:
          self.walkingOffsetX += 3
        if self.walkingOffsetY > 0:
          self.walkingOffsetY -= 3
        elif self.walkingOffsetY < 0:
          self.walkingOffsetY += 3
      if self.direction == RIGHT:
        if self.walkingOffsetX > - self.maxDistance:
          self.walkingOffsetX -= 3
        if self.walkingOffsetY > 0:
          self.walkingOffsetY -= 3
        elif self.walkingOffsetY < 0:
          self.walkingOffsetY += 3
      if self.walkingOffsetX == -6:
        self.walkingOffsetX = -3
      if self.walkingOffsetX == 6:
        self.walkingOffsetX = 3
      if self.walkingOffsetY == -6:
        self.walkingOffsetY = -3
      if self.walkingOffsetY == 6:
        self.walkingOffsetY = 3
    
  def update(self, x, y, game):
    self.x = (level.WIDTH - 96) / 2
    self.y = (level.HEIGHT - 96) / 2
    
    if self.isNPC == False:
      if game.mykeys.keyUp == True:
        self.isWalking = True
        self.direction = UP
        self.walkDirection = UP
      if game.mykeys.keyDown == True:
        self.isWalking = True
        self.direction = DOWN
        self.walkDirection = DOWN
      if game.mykeys.keyLeft == True:
        self.isWalking = True
        self.direction = LEFT
        self.walkDirection = LEFT
      if game.mykeys.keyRight == True:
        self.isWalking = True
        self.direction = RIGHT
        self.walkDirection = RIGHT
        
    if self.name == "Melody" or self.name == "Moonfire":
      if game.mykeys.keyUp == True:
        self.isWalking = True
        if self.walkingOffsetX < 0:
          self.walkDirection = RIGHT
        elif self.walkingOffsetX > 0:
          self.walkDirection = LEFT
        else:
          self.walkDirection = UP
      if game.mykeys.keyDown == True:
        self.isWalking = True
        if self.walkingOffsetX < 0:
          self.walkDirection = RIGHT
        elif self.walkingOffsetX > 0:
          self.walkDirection = LEFT
        else:
          self.walkDirection = DOWN
      if game.mykeys.keyLeft == True:
        self.isWalking = True
        if self.walkingOffsetY < 0:
          self.walkDirection = DOWN
        elif self.walkingOffsetY > 0:
          self.walkDirection = UP
        else:
          self.walkDirection = LEFT
      if game.mykeys.keyRight == True:
        self.isWalking = True
        if self.walkingOffsetY < 0:
          self.walkDirection = DOWN
        elif self.walkingOffsetY > 0:
          self.walkDirection = UP
        else:
          self.walkDirection = RIGHT
        
    if self.isWalking == True:
      if self.direction == UP and game.mykeys.keyUp == False:
        self.isWalking = False
        self.animationFrame = 0
      if self.direction == DOWN and game.mykeys.keyDown == False:
        self.isWalking = False
        self.animationFrame = 0
      if self.direction == LEFT and game.mykeys.keyLeft == False:
        self.isWalking = False
        self.animationFrame = 0
      if self.direction == RIGHT and game.mykeys.keyRight == False:
        self.isWalking = False
        self.animationFrame = 0
      self.updateWalkingOffsets()
     
  def render(self, screen):
    if self.isWalking == True:
      frame = int(self.animationFrame / 4) # SLOWS DOWN ANIMATION BY 4
      if self.walkDirection == LEFT:
        screen.blit(self.walkLeftAnimation[frame], (self.x + self.walkingOffsetX, self.y + self.walkingOffsetY))
      if self.walkDirection == UP:
        screen.blit(self.walkUpAnimation[frame], (self.x + self.walkingOffsetX, self.y + self.walkingOffsetY))
      if self.walkDirection == RIGHT:
        screen.blit(self.walkRightAnimation[frame], (self.x + self.walkingOffsetX, self.y + self.walkingOffsetY))
      if self.walkDirection == DOWN:
        screen.blit(self.walkDownAnimation[frame], (self.x + self.walkingOffsetX, self.y + self.walkingOffsetY))
      self.animationFrame += 1
      if self.animationFrame > 20:
        self.animationFrame = 0
    else:
      if self.walkDirection == LEFT:
        screen.blit(self.idleLeft, (self.x + self.walkingOffsetX, self.y + self.walkingOffsetY))
      if self.walkDirection == UP:
        screen.blit(self.idleUp, (self.x + self.walkingOffsetX, self.y + self.walkingOffsetY))
      if self.walkDirection == RIGHT:
        screen.blit(self.idleRight, (self.x + self.walkingOffsetX, self.y + self.walkingOffsetY))
      if self.walkDirection == DOWN:
        screen.blit(self.idleDown, (self.x + self.walkingOffsetX, self.y + self.walkingOffsetY))
