
try:
  import sys
  import pygame
  import data
  import os.path
  from scene import Scene
  from player import Player
  from splashscene import SplashScene
  from menuscene import MenuScene
  from worldscene import WorldScene
  from gamemenuscene import GameMenuScene
  from battlescene import BattleScene
  from sounds import Sounds
  from keys import Keys
except ImportError, err:
  print "couldn't load module. %s" % (err)
  sys.exit(2)

SPLASH_SCENE_NUMBER = 0
MENU_SCENE_NUMBER = 1
WORLD_SCENE_NUMBER = 2
GAME_MENU_SCENE_NUMBER = 3
BATTLE_SCENE_NUMBER = 4

class Game:
    def __init__(self, fullscreen):
  	  self.initialize(fullscreen)
  	  self.update()
    	
    def initialize(self, fullscreen):
      self.running = 1
      self.fullscreen = fullscreen
      self.sceneNumber = BATTLE_SCENE_NUMBER
      
      # Initialize screen
      pygame.init()
      self.screenWidth = 640
      self.screenHeight = 480
      if self.fullscreen:
	    self.screen = pygame.display.set_mode((self.screenWidth, self.screenHeight), pygame.FULLSCREEN|pygame.HWSURFACE|pygame.DOUBLEBUF, 32)
      else:
	    self.screen = pygame.display.set_mode((self.screenWidth, self.screenHeight), pygame.HWSURFACE|pygame.DOUBLEBUF, 32)
      pygame.display.set_caption('The Mysterious Tower')
        
	  # Initialize assets
      self.player = Player(500, 500)
      self.splashscene = SplashScene(self)
      self.menuscene = MenuScene(self)
      
      # Load previous game if any
      if os.path.exists("data/save.game.sav"):
        try:
          self.loadGame()
        except IOError:
          self.day = 1
      else:
        self.day = 1
      
      self.worldscene = WorldScene(self, self.day)
      self.gamemenuscene = GameMenuScene(self)
      self.battlescene = BattleScene(self)
      self.mysounds = Sounds()
      self.mykeys = Keys()
      
      
      # Initialize clock
      self.clock = pygame.time.Clock()
      return
    
    def update(self):
      while self.running:
        # Make sure game doesn't run at more than 60 frames per second
        self.clock.tick(60)
        self.mykeys.update()
        if self.sceneNumber == SPLASH_SCENE_NUMBER:
          self.splashscene.update(self)
          self.splashscene.render()
        elif self.sceneNumber == MENU_SCENE_NUMBER:
          self.menuscene.update(self)
          self.menuscene.render()
        elif self.sceneNumber == WORLD_SCENE_NUMBER:
          self.worldscene.update(self)
          self.worldscene.render()
        elif self.sceneNumber == GAME_MENU_SCENE_NUMBER:
          self.gamemenuscene.update(self)
          self.gamemenuscene.render(self)
        elif self.sceneNumber == BATTLE_SCENE_NUMBER:
          self.battlescene.update(self)
          self.battlescene.render()
          
        # Quits the game when escape is pressed
        if self.mykeys.keyEscape == True:
          pygame.quit()
          self.running = False
          pygame.quit()
    
    def loadGame(self):
      file = data.load('game.sav', 'rb')
      self.day = int(file.readline())
      self.player.gold = int(file.readline())
      self.player.rosewind.level = int(file.readline())
      self.player.rosewind.exp = int(file.readline())
      self.player.melody.level = int(file.readline())
      self.player.melody.exp = int(file.readline())
      self.player.moonfire.level = int(file.readline())
      self.player.moonfire.exp = int(file.readline())
      self.player.items.rosewindWeapon = int(file.readline())
      self.player.items.rosewindAccessory = int(file.readline())
      self.player.items.melodyWeapon = int(file.readline())
      self.player.items.melodyAccessory = int(file.readline())
      self.player.items.moonfireWeapon = int(file.readline())
      self.player.items.moonfireAccessory = int(file.readline())
      self.player.items.sword1 = int(file.readline())
      self.player.items.sword2 = int(file.readline())
      self.player.items.sword3 = int(file.readline())
      self.player.items.sword4 = int(file.readline())
      self.player.items.sword5 = int(file.readline())
      self.player.items.sword6 = int(file.readline())
      self.player.items.instrument1 = int(file.readline())
      self.player.items.instrument2 = int(file.readline())
      self.player.items.instrument3 = int(file.readline())
      self.player.items.instrument4 = int(file.readline())
      self.player.items.instrument5 = int(file.readline())
      self.player.items.book1 = int(file.readline())
      self.player.items.book2 = int(file.readline())
      self.player.items.book3 = int(file.readline())
      self.player.items.book4 = int(file.readline())
      self.player.items.accessory1 = int(file.readline())
      self.player.items.accessory2 = int(file.readline())
      self.player.items.accessory3 = int(file.readline())
      self.player.items.accessory4 = int(file.readline())
      self.player.items.accessory5 = int(file.readline())
      self.player.items.accessory6 = int(file.readline())
      self.player.items.accessory7 = int(file.readline())
      self.player.items.consumable1 = int(file.readline())
      self.player.items.consumable2 = int(file.readline())
      self.player.items.consumable3 = int(file.readline())
      self.player.items.consumable4 = int(file.readline())
      file.close()
    
    def switchToMenu(self):
      self.sceneNumber = MENU_SCENE_NUMBER
    
    def switchToWorld(self):
      self.sceneNumber = WORLD_SCENE_NUMBER

    def switchToGameMenu(self):
      self.sceneNumber = GAME_MENU_SCENE_NUMBER
    
    def switchToBattleScene(self):
      self.sceneNumber = BATTLE_SCENE_NUMBER
 
