
try:
  import sys
  import pygame
  import data
  import graphics
  import items
  from level import Level
  from character import Character
  from items import Items
except ImportError, err:
  print "couldn't load module. %s" % (err)
  sys.exit(2)

class Player:
  MOVESPEED = 5

  def __init__(self, x, y):
    self.x = x
    self.y = y
    self.hasRosewind = True
    self.hasMelody = True
    self.hasMoonfire = True
    self.gold = 50
    self.rosewind = Character("Rosewind", self.x, self.y)
    self.melody = Character("Melody", self.x, self.y)
    self.moonfire = Character("Moonfire", self.x, self.y)
    self.items = Items()
    self.escapePercentage = 30
    self.escapePercentageBoost = 0
  
  def isPartyDead(self):
    if self.rosewind.hp > 0 or self.melody.hp > 0 or self.moonfire.hp > 0:
      return False
    else:
      return True
  
  def applyItemEffects(self):
    # Initialize effects
    self.escapePercentageBoost = 0
    
    self.rosewind.boostHp = 0
    self.rosewind.boostMp = 0
    self.rosewind.boostAtk = 0
    self.rosewind.boostMag = 0
    self.rosewind.restoredHPAfterFight = 0
    self.rosewind.restoredMPAfterFight = 0
    
    self.melody.boostHp = 0
    self.melody.boostMp = 0
    self.melody.boostAtk = 0
    self.melody.boostMag = 0
    self.melody.restoredHPAfterFight = 0
    self.melody.restoredMPAfterFight = 0
    
    self.moonfire.boostHp = 0
    self.moonfire.boostMp = 0
    self.moonfire.boostAtk = 0
    self.moonfire.boostMag = 0
    self.moonfire.restoredHPAfterFight = 0
    self.moonfire.restoredMPAfterFight = 0
  
    # Rosewind weapons
    if self.items.rosewindWeapon == items.STICK:
      self.rosewind.boostAtk += 1
    elif self.items.rosewindWeapon == items.DAGGER:
      self.rosewind.boostAtk += 2
    elif self.items.rosewindWeapon == items.SWORD:
      self.rosewind.boostAtk += 5
    elif self.items.rosewindWeapon == items.KATANA:
      self.rosewind.boostAtk += 7
    elif self.items.rosewindWeapon == items.CUTLASS:
      self.rosewind.boostAtk += 8
      self.rosewind.boostMag += 3
    elif self.items.rosewindWeapon == items.BIG_FAT_SWORD:
      self.rosewind.boostAtk += 12
    # Melody weapons
    if self.items.melodyWeapon == items.WHISTLE:
      self.melody.boostMag += 1
    elif self.items.melodyWeapon == items.BELL:
      self.melody.boostAtk+=1
      self.melody.boostMag+=2
    elif self.items.melodyWeapon == items.FLUTE:
      self.melody.boostMag+=5
    elif self.items.melodyWeapon == items.HARP:
      self.melody.boostMag+=10
    elif self.items.melodyWeapon == items.SAXOPHONE:
      self.melody.boostAtk+=5
      self.melody.boostMag+=8
    # Moonfire weapons
    if self.items.moonfireWeapon == items.RAINBOW_BOOK:
      self.moonfire.boostAtk+=1
    elif self.items.moonfireWeapon == items.SKULL_BOOK:
      self.moonfire.boostAtk+=2
      self.moonfire.boostMag+=2
    elif self.items.moonfireWeapon == items.SPARKLES_BOOK:
      self.moonfire.boostAtk+=3
      self.moonfire.boostMag+=8
    elif self.items.moonfireWeapon == items.LIGHTNING_BOOK:
      self.moonfire.boostMag+=15
    # Team effet accessory (cape)
    if self.items.rosewindAccessory == items.CAPE or self.items.melodyAccessory == items.CAPE or self.items.moonfireAccessory == items.CAPE:
      self.escapePercentageBoost+=45
    # Rosewind accessory
    if self.items.rosewindAccessory == items.SADDLE:
      self.rosewind.boostHp+=25
    elif self.items.rosewindAccessory == items.RIBBON:
      self.rosewind.restoredHPAfterFight = 5
    elif self.items.rosewindAccessory == items.SMALL_NECKLACE:
      self.rosewind.boostAtk+=3
      self.rosewind.boostMag+=2
    elif self.items.rosewindAccessory == items.BIG_NECKLACE:
      self.rosewind.restoredMPAfterFight = 5
    elif self.items.rosewindAccessory == items.RING:
      self.rosewind.boostAtk += 5
    elif self.items.rosewindAccessory == items.HAT:
      self.rosewind.boostMag += 5
    # Melody accessory
    if self.items.melodyAccessory == items.SADDLE:
      self.melody.boostHp+=25
    elif self.items.melodyAccessory == items.RIBBON:
      self.melody.restoredHPAfterFight = 5
    elif self.items.melodyAccessory == items.SMALL_NECKLACE:
      self.melody.boostAtk+=3
      self.melody.boostMag+=2
    elif self.items.melodyAccessory == items.BIG_NECKLACE:
      self.melody.restoredMPAfterFight = 5
    elif self.items.melodyAccessory == items.RING:
      self.melody.boostAtk += 5
    elif self.items.melodyAccessory == items.HAT:
      self.melody.boostMag += 5
    # Moonfire accessory
    if self.items.moonfireAccessory == items.SADDLE:
      self.moonfire.boostHp += 25
    elif self.items.moonfireAccessory == items.RIBBON:
      self.moonfire.restoredHPAfterFight = 5
    elif self.items.moonfireAccessory == items.SMALL_NECKLACE:
      self.moonfire.boostAtk += 3
      self.moonfire.boostMag += 2
    elif self.items.moonfireAccessory == items.BIG_NECKLACE:
      self.moonfire.restoredMPAfterFight = 5
    elif self.items.moonfireAccessory == items.RING:
      self.moonfire.boostAtk += 5
    elif self.items.moonfireAccessory == items.HAT:
      self.moonfire.boostMag += 5
    
    self.rosewind.maxHp = self.rosewind.baseMaxHp + self.rosewind.boostHp
    self.rosewind.maxMp = self.rosewind.baseMaxMp + self.rosewind.boostMp
    self.rosewind.atk = self.rosewind.baseAtk + self.rosewind.boostAtk
    self.rosewind.mag = self.rosewind.baseMag + self.rosewind.boostMag
    
    self.melody.maxHp = self.melody.baseMaxHp + self.melody.boostHp
    self.melody.maxMp = self.melody.baseMaxMp + self.melody.boostMp
    self.melody.atk = self.melody.baseAtk + self.melody.boostAtk
    self.melody.mag = self.melody.baseMag + self.melody.boostMag
    
    self.moonfire.maxHp = self.moonfire.baseMaxHp + self.moonfire.boostHp
    self.moonfire.maxMp = self.moonfire.baseMaxMp + self.moonfire.boostMp
    self.moonfire.atk = self.moonfire.baseAtk + self.moonfire.boostAtk
    self.moonfire.mag = self.moonfire.baseMag + self.moonfire.boostMag
    
  def update(self, game):
    self.rosewind.update(self.x, self.y, game)
    if self.hasMelody == True:
      self.melody.update(self.x, self.y, game)
    if self.hasMoonfire == True:
      self.moonfire.update(self.x, self.y, game)
  
  def render(self, screen):
    if self.hasMelody == True and self.melody.walkingOffsetY < 0:
      if self.hasMoonfire == True:
        self.moonfire.render(screen)
      if self.hasMelody == True:
        self.melody.render(screen)
      self.rosewind.render(screen)
    else:
      self.rosewind.render(screen)
      if self.hasMelody == True:
        self.melody.render(screen)
      if self.hasMoonfire == True:
        self.moonfire.render(screen)
